! ************************************************************************
!  E.S.O. - 3.60 Upgrade Project
! 
!  who		when		what
!  ----------	--------	------------------------
!  c.levin	920106		created
!  p.leisy	980806		VLT Software adaptation
!  kbanse	100107		implement verifications
!  kbanse	100407		last modif
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++
!    NAME:	EXTPoint
!  
!    SYNOPSIS:	EXTPoint X_slit_pos Y_slit_pos Angle Method
!    
!    DESCRIPTION:	
!    Computes the Offsets to be applied to the Telescope
!    ???????   It is called from the EFOSClib.tcl trough PCO. 
!     
!    FILES:	
! 
!    ENVIRONMENT:	
!    PCO must be running together with a background MIDAS session	
! 
!    RETURN VALUES:
!    Offset in Alpha is written in the keyword OUTPUTR(1)
!    Offset in Delta is written in the keyword OUTPUTR(2)
! 
! ------------------------------------------------------------------------
! PARAMETERS:
! p1 --> x-position of the slit
! p2 --> y-position of the slit
! p3 --> Rotator angle (default=0).
! p4 --> C[URSOR] or G[AUSS] (default=G)
!
! SYSTEM DEPENDENCIES:
! It calls the Unix command "sleep"
!
! EXTERNAL VARIABLES:
! From get_mode.prg: obsmode, imscale, ptoffset
!
! ------------------------------------------------------
defi/par p1 ? N "Enter X position of slit: "
defi/par p2 ? N "Enter Y position of slit: "
defi/par p3 ? N "Enter the current Rotator Angle: "
defi/par p4 G C "Centering Method Cursor or [G]auss: "
!
defi/loc ycen/r/1/1	0.0	! target y coordinate
defi/loc xcen/r/1/1	0.0	! target x coordinate
defi/loc dx/r/1/1	0.0	! target x offset
defi/loc dy/r/1/1	0.0	! target y offset
defi/loc ptangle/r/1/1	{P3}
defi/loc ra/r/1/1	0.0
defi/loc dec/r/1/1	0.0
defi/loc alfa/r/1/1	0.0
defi/loc salfa/r/1/1	0.0
defi/loc calfa/r/1/1	0.0
defi/loc teta/r/1/1	0.0
defi/loc steta/r/1/1	0.0
defi/loc cteta/r/1/1	0.0
defi/loc c1/r/1/1	0.0
defi/loc c2/r/1/1	0.0
defi/loc xoff/r/1/1	0.0
defi/loc yoff/r/1/1	0.0
defi/loc obsmode/c/1/4	0
!
! sleep for a while because a lack of synchronization
! between graphical interface and center/gauss
$ sleep 2
! 
clear/chan ov
set/cur ? c_hair
if p4(1:1) .eq. "G" then
!   center/gauss cursor pointing
    center/gauss {idimemc},moveReference pointing   
    if outputi(1) .eq. 0 then
        write/out
        write/out "Invalid number of coordinates ..."
        return
    endif
    if outputi(1) .gt. 1 then
        write/out
        write/out "Take care, more than 1 centering done ... taking the last one"
    endif
!    xcen = {pointing,:xcen,1}
!    ycen = {pointing,:ycen,1}
    xcen = outputr(5)
    ycen = outputr(6)
else
    get/cursor
    if outputi(1) .eq. 0 then
        write/out
        write/out Invalid number of coordinates.
        return
    endif
    xcen = outputr(12)
    ycen = outputr(13)
endif
! 
dx = {xcen} - {p1}
dy = {ycen} - {p2}

outputr(1) = outputr(5)
outputr(2) = outputr(6)
outputr(3) = outputr(7)
outputr(4) = outputr(8)
outputr(5) = outputr(12)
outputr(6) = outputr(13)
outputr(7) = dx
outputr(8) = dy

write/keyw -
rcorr  1001.6193,830.65875,0.71167E-01,0.49522E-01,10.150,8.0401,23644.
write/keyw rcorr/r/7/2  1.62000E+00,-1.69341E+02
@@ kcompare point010:rcorr outputr 1,8 0.001
! 
obsmode = "EFO"
!
write/out "poloff={poloff}  ptoffset={ptoffset}"
outputr(1) = poloff
outputr(2) = ptoffset
write/keyw rcorr 1.54000E+00,-9.00000E+01
@@ kcompare point020:rcorr outputr 1,2 0.0001

poloff = 0
ptoffset = 0
write/out "poloff={poloff}  ptoffset={ptoffset}"
if obsmode(1:2) .eq. "EF" then
    alfa = {p3} + {poloff}
    write/out "Observing mode : EFOSC"
else
    alfa  = {p3} + ptoffset + {poloff}
endif

salfa = m$sin(alfa)
calfa = m$cos(alfa)
teta = {alfa} 				!what's that??
comp/key steta = M$SIN('teta')
comp/key cteta = M$COS('teta')

outputr(1) = alfa
outputr(2) = salfa
outputr(3) = calfa
write/keyw rcorr 0.,0.,1.
@@ kcompare point040:rcorr outputr 1,3 0.0001

if obsmode(1:3) .eq. "EFO" then		! EFOSC2
!   ra  = imscale*(dx*salfa + dy*calfa)	! New TCS orientation 25/01/98
!   dec = imscale*(dx*calfa - dy*salfa) 
    dec = imscale*(-dx*salfa + dy*calfa)	! New TCS orientation 25/01/98
    ra  = imscale*( dx*calfa + dy*salfa) 
!    xoff = 54.928*dy
!    yoff = 54.928*dx
else if obsmode(1:3) .eq. "EFI" then		! EFOSC1
    ra  = -imscale*(dx*salfa - dy*calfa)	
    dec = -imscale*(dx*calfa + dy*salfa) 
!    ra   = imscale*( {cteta}*{dx} + {steta}*{dy} ) 
!    dec  = imscale*(-{steta}*{dx} + {cteta}*{dy} ) 
else
    return
endif

outputr(1) = ra
outputr(2) = dec
outputr(3) = imscale
write/keyw rcorr 1.94400E-01,-2.03209E+01,0.12
@@ kcompare point050:rcorr outputr 1,3 0.001

write/out " "
set/format  F7.2
if obsmode(1:2) .eq. "EF" then
    write/out   Rotator angle: {ptangle} degrees
endif
write/out   "Offset : X = {dx} pixels           Y = {dy} pixels   "
write/out   "            imscale = {imscale}                      "
write/out   "*****************************************************"
write/out   "Offsets:  RA = {ra} arcsecs     DEC = {dec} arcsecs"
write/out   "*****************************************************"
write/out
! ****************************
! output for Templates
! ****************************
outputR(1) = {ra}
outputR(2) = {dec}

clear/chan ov
 
